/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2011 by Luca Bellonda and individual contributors       *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/

#ifndef QXMLEDITWIDGET_COMPAREDIALOG_H
#define QXMLEDITWIDGET_COMPAREDIALOG_H

#include <QDialog>
#include "libQXmlEdit_global.h"
#include "regola.h"

namespace Ui
{
class Compare;
}

class LIBQXMLEDITSHARED_EXPORT Compare : public QDialog
{
    Q_OBJECT

    enum EDiffStates { DF_MISSING, DF_EQUAL, DF_DIFFERENT, DF_ADDED };
public:
    Compare(Regola *original, QWidget *parent = 0);
    ~Compare();

    static void doCompare(QWidget *parent, Regola *original);

    bool loadRule(const QString &filePath);
    bool loadRule(QByteArray dataIn);
    bool loadData();
    bool loadData(const QString &filePath);
    bool loadData(QByteArray dataIn);
    bool areDifferent();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::Compare *ui;
    QDomDocument document;
    Regola *original;
    Regola *toCompare;
    QString resultText;
    bool ignoreText;
    bool compareOnlyElements;
    bool showOldValues;
    bool _areDifferent;

    void endElement(const int indent, Element *element, const EDiffStates state);
    void dumpElement(const int indent, Element *element, const EDiffStates state);
    void recurse(QVector<Element*> &origElements, QVector<Element*> &compareElements, const int indent);
    void recurseAll(Element* element, const EDiffStates state, const int indent);
    void doCompare();
    void compareAttributes(Element *orig, Element *compare);
    void insertAttrInHash(QHash<QString, Attribute*> &hash, Element *target);
    void dumpAttr(Attribute *attr, const EDiffStates state);
    void closeTag(Element *element, const EDiffStates state);
    void filterElements(QVector<Element*> &origElements, QVector<Element*> &destElements, const bool isFilter);
    void dumpAttributes(Element *element, const EDiffStates state);
    void dumpText(const QString text, const EDiffStates state, const int indent);
    void setDifference();
    bool compareTwoText(const QString &str1, const QString &str2);

    bool loadFile(const QString &filePath);

    static QString convertTextInHTML(const QString strIn);

private slots:
    void accept();
    void on_copyToClipboard_clicked();

};

#endif // QXMLEDITWIDGET_COMPAREDIALOG_H
