/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2011 by Luca Bellonda and individual contributors       *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/

#include <QApplication>
#include "xmlEdit.h"
#include "xmleditglobals.h"
#include "config.h"
#include "mainwindow.h"
#include "element.h"
#include "utils.h"

#if !defined(QXMLEDIT_NOMAIN)

const char *APP_TITLE = QT_TR_NOOP("QXmlEdit");

#define QXMLEDIT_TRANSLATIONS_PREFIX "QXmlEdit_"

static bool decodeCommandLine(QApplication &app, QString &fileName);
static void startTanslator(QApplication *app);

static QTranslator qtLibTranslator;
static QTranslator qXmlEditTranslator;

int main(int argc, char *argv[])
{
    QString fileName = "";

    QApplication app(argc, argv);
    Q_INIT_RESOURCE(risorse);

    QCoreApplication::setOrganizationName(ORGANIZATION_NAME);
    QCoreApplication::setOrganizationDomain(ORGANIZATION_DOMAIN);
    QCoreApplication::setApplicationName(APPLICATION_NAME);
    QXmlEditGlobals::setAppTitle(APP_TITLE);

    ApplicationData appData;
    if(!Config::init()) {
        Utils::errorReadingUserSettings();
    }
    appData.init();
    startTanslator(&app);

    Element::loadIcons();
    app.setWindowIcon(QIcon(":/tree/icon.png"));

    decodeCommandLine(app, fileName);
    MainWindow mainWindow(false, &app, &appData);

    mainWindow.show();
    if(!fileName.isEmpty()) {
        mainWindow.loadFile(fileName);
    } else {
        mainWindow.triggersWelcomeDialog();
    }
    int result = app.exec();
    appData.end();
    if(!Config::end()) {
        Utils::errorSavingUserSettings();
    }
    return result;
}


static bool decodeCommandLine(QApplication &app, QString &fileName)
{
    fileName = "" ;
    QStringList args = app.arguments();
    if(args.size() > 1) {
        fileName = args.at(1);
        if(!fileName.isEmpty()) {
            return true;
        }
    }
    return false;
}

void msgBoxDebug(const QString message)
{
    QMessageBox msgBox;
    msgBox.setText(message);
    msgBox.exec();
}


static void startTanslator(QApplication *app)
{
    if(qtLibTranslator.load("qt_" + QLocale::system().name(),
                            QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
        app->installTranslator(&qtLibTranslator);
    }
    if(qXmlEditTranslator.load(QXMLEDIT_TRANSLATIONS_PREFIX + QLocale::system().name(), ApplicationData::getResourceDir() + QString("/") + QString(STRINGIZE(TRANLASTION_DIR)))) {
        app->installTranslator(&qXmlEditTranslator);
    }
}

#endif //QXMLEDIT_NOMAIN
